#include <nspireio.h>
#include "HighScoreLib/include/HighScoreLib.h"
#include "Tests.h"

int StartsWith(const char* pstr, const char* pstrFind)
{
   if( strlen(pstrFind) > strlen(pstr) )
      return 0;

   int i;
   for(i=0; i<strlen(pstrFind); i++)
      if( tolower(pstr[i]) != tolower(pstrFind[i]) )
         return 0;

   return 1;
}

int simple_atoi(char* str)
{
	printf("Input string [%s]\n", str);
	int i;
	int num = 0;
	int digits = strlen(str);
	int magnitude = 1;
	for(i = (digits-1); i>=0; i--) {
		num += (str[i] - 48) * magnitude;
		magnitude *= 10;
	}
	return num;
}

int main(int argc, char *argv[])
{
	wait_no_key_pressed();
	
	// Initialize console 1.
	nio_console c1;
	
	nio_init(&c1,NIO_MAX_COLS,NIO_MAX_ROWS,0,0,NIO_COLOR_BLACK,NIO_COLOR_WHITE,TRUE);
	nio_set_default(&c1);
	nio_fflush(&c1);

	HighScoreLib highscore;
	HighScoreCreate(&highscore);
	HSSetCurrentDirectory(argv[0]);
	
	// Just showing printf
	nio_printf("%s built at %s, %s\n",__FILE__,__DATE__,__TIME__);
	nio_printf("For help with this console, type \"help\"!\n\n");
	while(1)
	{
		char text[100];
		nio_printf("> ");
		// If no text was entered, exit
		if(!nio_gets(text))
			continue;
		
		// Check for interesting text
		if (!strcmp(text, "help")) {
			nio_printf("HighScoreLib Test Help\n");
			nio_printf("  setcursor cursor\n");
			nio_printf("    Set the cursor type.\n");
			nio_printf("    cursor can be from 0-3, where:\n");
			nio_printf("      0 is a block cursor\n");
			nio_printf("      1 is an underscore cursor\n");
			nio_printf("      2 is a verical bar cursor\n");
			nio_printf("      3 is a custom cursor\n");
			nio_printf("  setcursorwidth\n");
			nio_printf("    Set the cursor width.\n");
			nio_printf("  add <score> <name>\n");
			nio_printf("    Adds an item to the high score table.\n");
			nio_printf("  list\n");
			nio_printf("    List items in the high score table.\n");
			nio_printf("  GetCount\n");
			nio_printf("    Get count of items in high score table.\n");
			nio_printf("  clear\n");
			nio_printf("    Clear the console.\n");
			nio_printf("  GetLimit\n");
			nio_printf("    Gets the limit of highscore to manage.\n");
			nio_printf("  exit\n");
			nio_printf("    Exit this console.\n");
		}

                if (!strcmp(text, "setcursor 0")) {
                        nio_cursor_type(&c1, NIO_CURSOR_BLOCK);
                        nio_printf(" Cursor type is now set to block cursor.\n");
                }
                if (!strcmp(text, "setcursor 1")) {
                        nio_cursor_type(&c1, NIO_CURSOR_UNDERSCORE);
                        nio_printf(" Cursor type is now set to underscore cursor.\n");
                }
                if (!strcmp(text, "setcursor 2")) {
                        nio_cursor_type(&c1, NIO_CURSOR_VERTICAL);
                        nio_printf(" Cursor type is now set to vertical bar cursor.\n");
                }

		if (!strcmp(text, "setcursorwidth")) {
                        nio_printf("Specify cursor width: ");
                        char num[10];
                        nio_gets(num);
                        nio_cursor_width(&c1, atoi(num));
                }

                if( StartsWith(text, "open") ) {
			int nRet = OpenHighScoreFile(highscore, text+5);
			nio_printf("Result of OpenHighScoreFile(%s) is: %d\n", text+5, nRet);
                }

		if( StartsWith(text, "GetCategory") ) {
			nio_printf("Current category identifier is: %s\n", GetCategoryID(highscore));
		}

		if( StartsWith(text, "SetCategory") ) {
			int nRet = SetCategoryID(highscore, text+12);
			nio_printf("SetCategoryID(%s) return with result: %d\n", text+12, nRet);
		}

		if( StartsWith(text, "ClearCategory") ) {
			int nRet = ClearCurrentCategoryID(highscore);
			nio_printf("ClearCategory returned with result: %d\n", nRet);
		}

		if( StartsWith(text, "add") ) {
			int nIndex = -1;
			char bufferNumber[64];
			int nBufferNumIndex = 0;
			char buffer[256];
			int nBufferIndex = 0;
			int nScore = 0;
			int i;

			int nSpace = 0;
			for(i=0; i<strlen(text); i++) {
				char ch = *(text + i);
				if( isspace(ch) ) {
					if( nSpace == 1 ) {
						bufferNumber[nBufferNumIndex] = '\0';
						nScore = atoi(bufferNumber);
					}
					nSpace++;
					continue;
				}
				if( isdigit(ch) && nSpace == 1 ) {
					bufferNumber[nBufferNumIndex++] = ch;
				}

				if( nSpace > 1 ) {
					buffer[nBufferIndex++] = ch;
					
				}
			}
			buffer[nBufferIndex] = '\0';

			int nRet = AddHighScore(highscore, NULL, nScore, buffer, 0, &nIndex, NULL);

			nio_printf("Result of AddHighScore is: %d\n", nRet);
			nio_printf("Item was place at index: %d\n", nIndex);
		}

		if( StartsWith(text, "GetCount")) {
			nio_printf("Count is %d items\n", GetNumberScores(highscore, NULL));
		}

		if ( StartsWith(text, "GetLimit")) {
			nio_printf("Limit is %d items\n", GetLimitNumber(highscore));
		}

		if( StartsWith(text, "SetLimit") ) {
			int nLimit = simple_atoi(text+9);
			nio_printf("Result of calling SetLimit(%d): %d\n", nLimit, SetLimitNumber(highscore, nLimit));
		}

		if( StartsWith(text, "ClearAllItems") ) {
			nio_printf("Result of calling ClearAllItems: %d\n", ClearAllItems(highscore, NULL));
		}

		if( StartsWith(text, "RemoveItem") ) {
			int nIndex = simple_atoi(text+11);
			nio_printf("Result of calling RemoveItem(%d): %d\n", nIndex, RemoveItem(highscore, NULL, nIndex));
		}

		if( StartsWith(text, "List") ) {
			nio_printf("Score\tName\tNumber\n");
			nio_printf("-----\t-----\t-----\n");
			int i;
			for(i=0; i<GetNumberScores(highscore, NULL); i++) {
				nio_printf("%d\t%s\t%d\n", 
					GetScore(highscore, NULL, i),
					GetName(highscore, NULL, i),
					GetNumber(highscore, NULL, i) );
			}
		}

		if( StartsWith(text, "RunTests") ) {
			RunTests();
		}
		
		if ( !strcmp(text, "clear")) {
			nio_clear(&c1);
		}
		
		if ( StartsWith(text, "exit")) break;
	}
	
	HighScoreFree(&highscore);
	nio_free(&c1);
	
	//if(has_colors)
	//	lcd_incolor();
		
	return 0;
}
