#include <nspireio.h>
#include "HighScoreLib/include/HighScoreLib.h"

#define PRINT_FUNC nio_printf("%s", __func__);

#define TEST_SUCCEEDED	(0)
#define TEST_FAILED	(1)

#define TEST_PERSISTING

int g_nRelocatedGlobals = 0;

const char* g_apstrSampleCategories[] = {"easy", "medium", "hard"};

int TestConstruction(HighScoreLib highscore)
{
   PRINT_FUNC;
   return TEST_SUCCEEDED;
}

int TestInitiallyEmpty(HighScoreLib highscore)
{
   PRINT_FUNC;
   return (GetNumberScores(highscore, NULL) == 0) ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestAdding1(HighScoreLib highscore)
{
   PRINT_FUNC;

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);

   return GetScore(highscore, NULL, 0) == 10 ? TEST_SUCCEEDED : TEST_FAILED;
}


int TestAdding2(HighScoreLib highscore)
{
   PRINT_FUNC;

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   return GetNumberScores(highscore, NULL) == 3 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestAddingWithID1(HighScoreLib highscore)
{
   PRINT_FUNC;

   const unsigned nEntries = 1;
   unsigned i,j;
   HSSetBatchMode(highscore, 1);
   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      for(j=0; j<nEntries; j++) {
         AddHighScore(highscore, g_apstrSampleCategories[i], rand(), "Any Name", 0, NULL, NULL);
      }
   }
   HSSetBatchMode(highscore, 0);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i]) != (int)nEntries )
         return TEST_FAILED;
   }

   return TEST_SUCCEEDED;
}

int TestRemoving1(HighScoreLib highscore)
{
   PRINT_FUNC;

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   RemoveItem(highscore, NULL, 1);

   return GetNumberScores(highscore, NULL) == 2 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestRemoving2(HighScoreLib highscore)
{
   PRINT_FUNC;

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   RemoveItem(highscore, NULL, 2);
   RemoveItem(highscore, NULL, 1);
   RemoveItem(highscore, NULL, 0);

   return GetNumberScores(highscore, NULL) == 0 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestClearAllItems(HighScoreLib highscore)
{
   PRINT_FUNC;
   
   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   ClearAllItems(highscore, NULL);

   return GetNumberScores(highscore, NULL) == 0 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestSetLimit1(HighScoreLib highscore)
{
   PRINT_FUNC;
   
   SetLimitNumber(highscore, 3);
   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 5, "fourth addition", 0, NULL, NULL);//Shouldn't be added

   return (GetNumberScores(highscore, NULL) == 3 && GetScore(highscore, NULL, 2) == 10) ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestIsHighScore1(HighScoreLib highscore)
{
   PRINT_FUNC;
   
   SetLimitNumber(highscore, 3);
   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   return IsHighScore(highscore, NULL, 5) == 0 ? TEST_SUCCEEDED : TEST_FAILED;//Shouldn't be since limit is 3 and this is not high enough
}

int TestIsHighScore2(HighScoreLib highscore)
{
   PRINT_FUNC;

   SetLimitNumber(highscore, 5);
   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   return IsHighScore(highscore, NULL, 5) == 1 ? TEST_SUCCEEDED : TEST_FAILED;//Should be; because limit is 5 this time
}

int TestIsHighScore3(HighScoreLib highscore)
{
   PRINT_FUNC;

   SetLimitNumber(highscore, 3);
   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   return IsHighScore(highscore, NULL, 25) == 1 ? TEST_SUCCEEDED : TEST_FAILED;//Should be; this is high enough score
}

int TestCategoryBasics(HighScoreLib highscore)
{
   PRINT_FUNC;

   AddHighScore(highscore, "easy", 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, "easy", 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, "hard", 15, "third addition", 0, NULL, NULL);

   return (GetNumberScores(highscore, "easy") == 2 && GetNumberScores(highscore, "hard") == 1) ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestCurrentCategory(HighScoreLib highscore)
{
   PRINT_FUNC;
   
   AddHighScore(highscore, "easy", 10, "first addition", 0, NULL, NULL);
   SetCategoryID(highscore, "hard");
   AddHighScore(highscore, "easy", 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   return (GetNumberScores(highscore, "easy") == 2 
        && GetNumberScores(highscore, "hard") == 1
        && strcmp(GetCategoryID(highscore), "hard")==0) ? TEST_SUCCEEDED : TEST_FAILED;
}

#ifdef TEST_PERSISTING
int TestCopyFile(HighScoreLib highscore)
{
   PRINT_FUNC;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");
   int nRet = AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   nRet = HSCopyFile("/documents/test.tns"/*From*/, "/documents/output.tns"/*To*/);
   return HIGHSCORE_OK == nRet ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestStress1(HighScoreLib highscore)
{
   PRINT_FUNC;

   const char* pstrFile = "/documents/test.tns";
   HSDeleteFile(pstrFile);//So start off clean
   OpenHighScoreFile(highscore, pstrFile);

   const unsigned nEntries = 1;
   unsigned i,j;
   HSSetBatchMode(highscore, HIGHSCORE_ENABLE_BATCH);
   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      for(j=0; j<nEntries; j++) {
         int nRes = AddHighScore(highscore, g_apstrSampleCategories[i], rand(), "Any Name", 0, NULL, NULL);
         if( HIGHSCORE_OK != nRes && HIGHSCORE_BATCHING_OK != nRes ) {
            printf("Wasn't able to add the %d item; error was %d\n", j, nRes);
         }
      }
   }
   int nRes = HSSetBatchMode(highscore, HIGHSCORE_DISABLE_BATCH);
   if( nRes != HIGHSCORE_OK ) {
      printf("Problems with disabling batch: %d\n", nRes);
      return TEST_FAILED;
   }

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != 0 )//Should always be initially empty
         return TEST_FAILED;
   }

   OpenHighScoreFile(highscore, pstrFile);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != (int)nEntries )
         return TEST_FAILED;
   }

   return TEST_SUCCEEDED;
}

int TestStress2(HighScoreLib highscore)
{
   PRINT_FUNC;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");

   const unsigned nEntries = 5;
   unsigned i,j;
   HSSetBatchMode(highscore, HIGHSCORE_ENABLE_BATCH);
   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      for(j=0; j<nEntries; j++) {
         int nRes = AddHighScore(highscore, g_apstrSampleCategories[i], rand(), "Any Name", 0, NULL, NULL);
         if( HIGHSCORE_OK != nRes && HIGHSCORE_BATCHING_OK != nRes ) {
            printf("Wasn't able to add the %d item; error was %d\n", j, nRes);
            return TEST_FAILED;
         }
      }
   }
   int nRes = HSSetBatchMode(highscore, HIGHSCORE_DISABLE_BATCH);
   if( nRes != HIGHSCORE_OK ) {
      printf("Problems with disabling batch: %d\n", nRes);
      return TEST_FAILED;
   }

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != 0 ) {//Should always be initially empty
         printf("After creating a HighScoreLib; all entries should be initially empty\n");
         return TEST_FAILED;
      }
   }

   OpenHighScoreFile(highscore, "/documents/test.tns");

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      int nScores = GetNumberScores(highscore, g_apstrSampleCategories[i] );
      if( nScores != (int)nEntries ) {
         printf("For category index %d; I expected %d items; but it has %d items\n", i, (int)nEntries, nScores);
         return TEST_FAILED;
      }
   }

   return TEST_SUCCEEDED;
}

int TestStress3(HighScoreLib highscore)
{
   PRINT_FUNC;

   const unsigned nEntries = 100;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");
   SetLimitNumber(highscore, 0);

   unsigned i,j;
   HSSetBatchMode(highscore, HIGHSCORE_ENABLE_BATCH);
   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      for(j=0; j<nEntries; j++) {
         int nValue = rand();
         int nRes = AddHighScore(highscore, g_apstrSampleCategories[i], nValue, "Any Name", 0, NULL, NULL);
         if( HIGHSCORE_OK != nRes && HIGHSCORE_BATCHING_OK != nRes ) {
            printf("Wasn't able to add the %d item of value %d; error was %d\n", j, nValue, nRes);
         }
      }
   }
   HSSetBatchMode(highscore, HIGHSCORE_DISABLE_BATCH);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != 0 ) {//Should always be initially empty
         printf("Category index %d should have %d items; but has %d items\n", i, 0, GetNumberScores(highscore, g_apstrSampleCategories[i] ));
         return TEST_FAILED;
      }
   }

   OpenHighScoreFile(highscore, "/documents/test.tns");

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != (int)nEntries ) {
         printf("Category index %d should have %d items; but has %d items\n", i, nEntries, GetNumberScores(highscore, g_apstrSampleCategories[i] ));
         return TEST_FAILED;
      }
   }

   return TEST_SUCCEEDED;
}

int TestStress4(HighScoreLib highscore)
{
   PRINT_FUNC;

   const unsigned nEntries = 1000;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");
   SetLimitNumber(highscore, nEntries);

   unsigned i,j;
   HSSetBatchMode(highscore, HIGHSCORE_ENABLE_BATCH);
   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      for(j=0; j<nEntries; j++) {
         AddHighScore(highscore, g_apstrSampleCategories[i], rand(), "Any Name", 0, NULL, NULL);
      }
   }
   HSSetBatchMode(highscore, HIGHSCORE_DISABLE_BATCH);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != 0 ) {//Should always be initially empty
         printf("Category index %d should have %d items; but has %d items\n", i, 0, GetNumberScores(highscore, g_apstrSampleCategories[i] ));
         return TEST_FAILED;
      }
   }

   OpenHighScoreFile(highscore, "/documents/test.tns");

   for(i=0; i<sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]); i++) {
      if( GetNumberScores(highscore, g_apstrSampleCategories[i] ) != (int)nEntries ) {
         printf("Category index %d should have %d items; but has %d items\n", i, nEntries, GetNumberScores(highscore, g_apstrSampleCategories[i] ));
         return TEST_FAILED;
      }
   }

   return TEST_SUCCEEDED;
}

int TestPersisting1(HighScoreLib highscore)
{
   PRINT_FUNC;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   if( GetNumberScores(highscore, NULL) != 0 )//Should always be initially empty
      return TEST_FAILED;

   OpenHighScoreFile(highscore, "/documents/test.tns");

   return GetNumberScores(highscore, NULL) == 3 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestPersisting2(HighScoreLib highscore)
{
   PRINT_FUNC;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   if( GetNumberScores(highscore, NULL) != 0 )//Should always be initially empty
      return 1;

   OpenHighScoreFile(highscore, "/documents/test.tns");

   AddHighScore(highscore, NULL, 35, "fourth addition", 0, NULL, NULL);

   return GetNumberScores(highscore, NULL) == 4 ? TEST_SUCCEEDED : TEST_FAILED;
}

int TestPersisting3(HighScoreLib highscore)
{
   PRINT_FUNC;

   HSDeleteFile("/documents/test.tns");//So start off clean
   OpenHighScoreFile(highscore, "/documents/test.tns");

   AddHighScore(highscore, NULL, 10, "first addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 20, "second addition", 0, NULL, NULL);
   AddHighScore(highscore, NULL, 15, "third addition", 0, NULL, NULL);

   HighScoreFree(&highscore);
   HighScoreCreate(&highscore);

   if( GetNumberScores(highscore, NULL) != 0 )//Should always be initially empty
      return 1;

   OpenHighScoreFile(highscore, "/documents/test.tns");

   RemoveItem(highscore, NULL, 1);

   return GetNumberScores(highscore, NULL) == 2 ? TEST_SUCCEEDED : TEST_FAILED;
}

#endif

typedef int (*testfunc)(HighScoreLib highscore);
   testfunc g_Tests[] =
   {
      TestConstruction,
      TestInitiallyEmpty,
      TestAdding1,
      TestAdding2,
      TestAddingWithID1,
      TestRemoving1,
      TestRemoving2,
      TestClearAllItems,
      TestSetLimit1,
      TestIsHighScore1,
      TestIsHighScore2,
      TestIsHighScore3,
      TestCategoryBasics,
      TestCurrentCategory,
#ifdef TEST_PERSISTING
      TestCopyFile,
      TestStress1,
      TestStress2,
      TestStress3,
      TestStress4,
      TestPersisting1,
      TestPersisting2,
      TestPersisting3,
#endif
   };

void RunTests()
{
   if( !g_nRelocatedGlobals ) {
      nl_relocdata((unsigned*)g_apstrSampleCategories, sizeof(g_apstrSampleCategories)/sizeof(g_apstrSampleCategories[0]));
      nl_relocdata((unsigned*)g_Tests, sizeof(g_Tests)/sizeof(g_Tests[0]));
      g_nRelocatedGlobals = 1;
   }

   unsigned i;
   for(i=0; i<sizeof(g_Tests)/sizeof(testfunc); i++) {
      HighScoreLib highscore;
      HighScoreCreate(&highscore);

      int nRet = g_Tests[i](highscore);
      if( nRet == TEST_SUCCEEDED ) {
         nio_printf("........OK\n");
      } else {
         nio_printf("........FAIL\n");
      }
      HighScoreFree(&highscore);
   }
}


